/**
 * Board class models the 9 by 9 rectangular array of squares.
 * Each square can be blank or have an integer value 1 through 9.
 * Each square also has a list of possible values that may be used
 * for that squares value.
 */
public class Board implements Sudoku
{
    // all squares on the Sudoku board
	private Square[][] square;
	// true if a square on the board has a new state; false otherwise
	private boolean stateChanged;
    // all sections that are defined for the board
	private Section[] section;
	
   /**
    * Initialize all data related to the board
    */
   public Board() {

	stateChanged = false; // indicates if a square has been changed	
	
	square = new Square[NUM_SECTIONS_PER_TYPE][NUM_SECTIONS_PER_TYPE];
	
	for (int r = 0; r < square.length; r++){
		for (int c = 0; c < square[0].length; c++){
		    // initializing a square with 0 means it has an
		    // undetermined value (possible values are 1 to 9)
			square[r][c] = new Square(0, r, c); 
		}
	  }
	  sectionsInit();
    }

  /*
   * Initialize all sections (rows, columns, and regions) so each
   * type of section is initialized with consecutive integer values
   * beginning with 0.
   */
  private void sectionsInit()
  {
	section = new Section[NUM_SECTIONS];
	for (int sec = 0; sec < NUM_SECTIONS; sec++)
	{
	  if ( sec < NUM_SECTIONS_PER_TYPE )
	    section[sec] = new Row( sec );
	  else if ( sec < 2 * NUM_SECTIONS_PER_TYPE )
	    section[sec] = new Column( sec - NUM_SECTIONS_PER_TYPE );
//	   section[sec] = new DiagonalTLBR( sec - NUM_SECTIONS_PER_TYPE );
	  else
	   section[sec] = new Region( sec - 2 * NUM_SECTIONS_PER_TYPE );
//	   section[sec] = new DiagonalTRBL( sec - 2 * NUM_SECTIONS_PER_TYPE );        
    }

    /* alternative initialization
	section = new Section[NUM_SECTIONS];
	for (int row = 0; row < NUM_ROWS; row++)
	{
	   section[row] = new Row( row );
	}
	for (int col = NUM_ROWS; col < NUM_ROWS + NUM_COLUMNS; col++)
	{
	   section[col] = new Column( col - NUM_ROWS );
	}
	for (int reg = NUM_ROWS + NUM_COLUMNS; reg < NUM_SECTIONS; reg++)
	{
	   section[reg] = new Region( reg - (NUM_ROWS + NUM_COLUMNS) );
	}
    */
  }

  /**
   * provides public access to private data to
   * simplifie code, but violates data hiding
   * Future Project: abstract squares array to provide
   * information without providing direct access to squares
   */
  public Square[][] getSquares()
  {
      return square;
  }
  
  /**
   * Sets the value of a square in the Sudoku board and
   * updates all sections that contain that square
   * @param row row of square whose state is changed
   * @param col column of square whose state is changed
   * @param val value used to set square value
   */
  public void updateSections(int row, int col, int val)
  {
     square[row][col].setValue(val);
     stateChanged = true;

	 for (int sec = 0; sec < NUM_SECTIONS; sec++){
       section[sec].update(square, square[row][col]);
	 }
  }

  /**
   * Examines all squares on the Sudoku board and updates
   * all squares that have been modified since the last update
   */
  public void updateBoard()
  {
    for (int r = 0; r < square.length; r++)
    {
	   for (int c = 0; c < square[0].length; c++)
	   {
		   if ( square[r][c].hasChanged() )
		   {
			  stateChanged = true;
		 	  updateSections( r, c, square[r][c].getValue() );
		      square[r][c].resetState();
		   }
	   }
    }
	checkForPairs();
  }

   public boolean hasChanged(){
 	   return stateChanged;
   }

   public void resetHasChanged(){
	   stateChanged = false;
   }

   // ******************************************
   private void checkForPairs(){
	 for (int sec = 0; sec < NUM_SECTIONS; sec++) //********
	 {
	     Section tempSection = section[sec];
	     int[] pair = tempSection.getPair(square);
	     if ( pair != null )
	       tempSection.processPair( square, pair );
	 }
   }
}
